<?php

namespace HCM\AVL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use HCM\AVL\Services\CalculoApuracaoAvaliacaoService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;

class CalculoApuracaoAvaliacaoController extends Crud {
    
    protected $CalculoApuracaoAvaliacaoService;
    protected $entityManager;
    
    
    public function __construct (Manager $dataSourceManager, HCMParameters $hcmParameters, CalculoApuracaoAvaliacaoService $calculoApuracaoAvaliacaoService){
        $this->hcmParameters = $hcmParameters;
        $this->CalculoApuracaoAvaliacaoService = $calculoApuracaoAvaliacaoService;
        parent::__construct($dataSourceManager);
    }
    
    public function determinaTipoApuracao(DTO\Request\Row $row, DTO\Response $response){
        try {
            $row = ($row->getRow()[0]);
            if($row['IDOPCAO']==='A'){
                self::iniciarApuracaoCompleta($row, $response);
            }else{
                self::iniciarApuracaoParcial($row, $response);
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function determinaTipoRetrocesso(DTO\Request\Row $row, DTO\Response $response){
        try {
            $row = ($row->getRow()[0]);
            if($row['IDOPCAO']==='A'){
                self::iniciarRetrocessoCompleto($row, $response);
            }else{
                self::iniciarRetrocessoParcial($row, $response);
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function iniciarApuracaoCompleta($row, DTO\Response $response){
        $nravaliacao = $row['NRAVALIACAO'];
        try{
            $totResultApur = $this->CalculoApuracaoAvaliacaoService->inicarApuracaoCompleta($nravaliacao, $response);
            if($totResultApur > 0 || $totResultApur < 0){
                $response->addNotification(new Notification("Resultado de {$totResultApur} colaborador(es) apurado(s)", Notification::TYPE_SUCCESS));
            }    
    }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function iniciarApuracaoParcial($row, DTO\Response $response){
        try{
            $qtColabs = $this->CalculoApuracaoAvaliacaoService->iniciarApuracaoParcial($row, $response);
            if($qtColabs >0){
                $response->addNotification(new Notification("Resultado de {$qtColabs} colaborador(es) apurado com sucesso", Notification::TYPE_SUCCESS));
            }else{
                $response->addNotification(new Notification("Colaborador(es) já apurados", Notification::TYPE_SUCCESS));
            }
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function iniciarRetrocessoCompleto($row, DTO\Response $response){
        $nravaliacao = $row['NRAVALIACAO'];
        try{
            $totResultRet = $this->CalculoApuracaoAvaliacaoService->iniciarRetrocessoCompleto($nravaliacao, $response);
            
            if($totResultRet != 0){
                $response->addNotification(new Notification("{$totResultRet} apuração(ões) retrocedidas com sucesso", Notification::TYPE_SUCCESS));
            }else{
                $response->addNotification(new Notification("Nenhum colaborador retrocedido, não há apurações pra essa avaliação", Notification::TYPE_ALERT));
            }
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function iniciarRetrocessoParcial($row, DTO\Response $response){
        try{
            $qtColabs = $this->CalculoApuracaoAvaliacaoService->iniciarRetrocessoParcial($row, $response);
            if($qtColabs != 0){
                $response->addNotification(new Notification("Resultado de {$qtColabs} colaborador(es) retrocedidos com sucesso", Notification::TYPE_SUCCESS));
            }
            
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
   
}