<?php

namespace HCM\FPA\Registro;

class RegistroMovimentacaoHomolognet extends RegistroHomolognet{


    const MOVIMENTACAO = 'Movimentacao';
    const MOVIMENTACOES = 'Movimentacoes';
    const MOTIVO = 'CDMotivo';
    const DATA_MOVIMENTACAO = 'DTMovimentacao';

    const MESES_MOVIMENTACOES = 84;

    private $_motivoMovimentacao;
    private $_dtMovimentacao;


    public function geraRegistroMovimentacao(){
        $this->_geraRegistro();
    }


    private function _geraRegistro(){

        parent::geraRegistro(null, null, null);

        if (empty($this->_motivoMovimentacao) === false) {

            $tag = self::MOVIMENTACAO;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, null, 0);

            // Campo 59 - Tamanho 03 - Motivo da Movimentacao
            $this->_atribuiMotivoMovimentacao();

            // Campo 60 - Tamanho - 08 - Data da Movimentacao
            $this->_atribuiDataMovimentacao();

            $tag = parent::CARACTER_FINALIZADOR;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, null, 0);

        }
    }

    
    private function _atribuiMotivoMovimentacao(){
        /*
         * · Utilizar codificação conforme tabela de movimentação da Caixa.[Ref.4]
         * · A ocorrência deste campo deverá ser de acordo com o número de movimentações do empregado dentro de um dado período.
         * · Deverão ser informados no mínimo os últimos 24 meses (contratos com mais de dois anos de vigência), no caso de
         *   todas as férias terem sido integralmente gozadas e todos os 13º Salários integralmente pagos.
         */
        if (empty($this->_motivoMovimentacao) === false){
            $tag = self::MOTIVO.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $this->_motivoMovimentacao, 3);
        }
        
    }


    private function _atribuiDataMovimentacao(){
        /*
         * · Informe de acordo com o formato: aaaammdd.
         * · Este campo terá sua ocorrência condicionada e correspondente ao campo de número 59 e neste caso será obrigatório.
         * · Este campo terá sua ocorrência condicionada e correspondente ao campo 59.
         * · Este campo será obrigatório se o campo 59 for informado. Para este caso, seu valor não poderá ser nulo.
         */
        if (empty($this->_dtMovimentacao) === false){
            $tag = self::DATA_MOVIMENTACAO.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $this->_dtMovimentacao, 8);
        }
    }


    public function setDtMovimentacao($dtMovimentacao){
        $dataMovimentacao = new TkDateTime($dtMovimentacao);
        $dataMovimentacao = $dataMovimentacao->toString(TkDateTime::YEAR.TkDateTime::MONTH.TkDateTime::DAY);

        $this->_dtMovimentacao = $dataMovimentacao;
    }

    
    public function setMotivoMovimentacao($motivoMovimentacao){
        $this->_motivoMovimentacao = $motivoMovimentacao;
    }


}
