<?php

namespace HCM\FPA\Registro\sefip;

use HCM\FPA\Registro\RegistroSEFIP;
use HCM\Util\DateUtil;
use HCM\Util\MsgCalculo;

class Registro30SEFIP extends RegistroSEFIP
{

    const REGISTRO_30 = '30';

    const SIM = 'S';
    const NAO = 'N';
    const SOCIO = '3';
    const AUTONOMO = '10';

    private $_pessoa;
    private $_vinculo;

    private $_dtadmissao;
    private $_dtrescisao;

    private $_tpInscricao;
    private $_nrInscricao;
    private $_tpInscricaoTomador;
    private $_nrInscricaoTomador;
    private $_nrRecolhimento;
    private $_nrCategoria;
    private $_competencia;
    private $_competencia13;
    private $_codMovimentacao;

    private $_base13;

    /**
     * Preparacao para a criacao do Registro 30.
     *
     * @param GpepessoaEntity $pessoa          Instancia de GpepessoaEntity.
     * @param GpeVinculomEntity $vinculo         Instancia de GpeVinculomEntity.
     * @param string $remuneracao     Valor da remuneracao do funcionario.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function geraRegistro30($pessoa, $vinculo, $remuneracao)
    {
        $this->_pessoa = $pessoa;
        $this->_vinculo = $vinculo;

        $this->_dtadmissao = DateUtil::adicionaIntervalo($vinculo->getDtadmissaovinc(), 15, DateUtil::DIA);
        
        if ($vinculo->getDtrescisaovinc()) {
            $this->_dtrescisao = DateUtil::getDataDeString($vinculo->getDtrescisaovinc());
        } else {
            $this->_dtrescisao = null;
        }

        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_competencia = substr($this->getCompetencia(), 5, 2);
        $this->_competencia13 = $this->getCompetencia13();
        
        if (!$this->_pessoa->historico->getDtnascpessoa() && $this->_vinculo->getNrtipovinculom() != self::SOCIO && $this->_vinculo->getNrtipovinculom() != self::AUTONOMO) {
            //@todo MsgCalculo_SEFIP
            MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $this->_pessoa->historico->getNmpessoa() . ' não possui data de nascimento.');
        }

        $this->_geraRegistro($remuneracao);

    }


    /**
     * Cria o Registro 30.
     *
     * @param string $remuneracao    Valor da remuneracao do funcionario.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    private function _geraRegistro($remuneracao)
    {
        // Bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_30, $this->_tpInscricao, $this->_nrInscricao);

        // Bloco 4 | 018 | Tipo de Inscricao do tomador.
        if (is_null($this->_tpInscricaoTomador)) {
            $this->atribuiDado(parent::CARACTER_BRANCO, 1, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        } else {
            $this->atribuiDado($this->_tpInscricaoTomador, 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

        // Bloco 5 | 019-032 | Inscricao do tomador.
        if (is_null($this->_tpInscricaoTomador)) {
            $this->atribuiDado(parent::CARACTER_BRANCO, 14, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        } else {
            $this->atribuiDado($this->_nrInscricaoTomador, 14, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }

        // Bloco 6 | 033-043 | PIS/PASEP/CI.
        // $identificador = '12345678912'; // pegar da pessoa.
        $identificador = $this->_pessoa->historico->getNrpispaseppes();
        $this->atribuiDado($identificador, 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 7 | 044-051 | Data de Admissao.
        $this->_atribuiDataAdmissao($this->_vinculo->getDtadmissaovinc());

        // Bloco 8 | 052-053 | Categoria do Trabalahador.
        /* @todo rever campos de categoria na situacao funcional e no vinculo */
        // $categoria = '01';.
        $this->atribuiDado($this->_nrCategoria, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 9 | 054-123 | Nome do trabalhador.
        $nome = $this->_pessoa->historico->getNmpessoa();
        $this->atribuiDado($nome, 70, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 10 | 124-134 | Matricula.
        $this->_atribuiMatricula($this->_vinculo->getCdmatricula());

        // Bloco 11 | 135-141 | Numero CTPS.        
        $this->_atribuiCTPS($this->_pessoa->historico->getNrctpspessoa());

        // Bloco 12 | 142-146 | Serie CTPS.
        $this->_atribuiSerieCTPS($this->_pessoa->historico->getNrseriectpspes());

        // Bloco 13 | 147-154 | Data de opcao do FGTS.
        // @todo - Normalmente eh a data de admissao do vinculo.
        $dataOpcao = $this->_vinculo->getDtadmissaovinc();
        $this->_atribuiDataOpcaoFGTS($dataOpcao);

        // Bloco 14 | 155-162 | Data de Nascimento.
        $this->_atribuiDataNascimento($this->_pessoa->historico->getDtnascpessoa());

        // Bloco 15 | 163-167 | Codigo Brasileiro de Ocupacao.
        //$cbo = '1234';
        $cbo = $this->_vinculo->cbo;
        $this->_atribuiCbo($cbo);

        // Bloco 16 | 168-182 | Remuneracao sem 13.
        /* Base de incidencia de INSS / FGTS.   Nao pode ser informado para competencia 13 */
        // Buscar a alteracao do gpealtsalario.
        $this->_atribuiRemuneracao($remuneracao[RegistroSEFIP::SEFIP_REM_SEM_13SAL]);

        // Bloco 17 | 183-197 | Remuneracao 13 - Parcela paga no mes.
        /* Adiantamento de 13 Salario(normal/ferias). Nao pode ser informado para competencia 13 */
        $this->_atribuiRemuneracao13($remuneracao[RegistroSEFIP::SEFIP_13SAL_PAGO_MES]);

        // Bloco 18 | 198-199 | Classe de contribuicao.
        /* Nao pode ser informado para competencia 13.
         * Campo obrigatorio para as categorias 14 e 16 (apenas em recolhimentos de competencias anteriores a 03/2000).
         * Nao pode ser informado para as demais categorias.
         */
        $classeContribuicao = parent::CARACTER_BRANCO;
        $this->atribuiDado($classeContribuicao, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 19 | 200-201 | Ocorrencia - exposicao a agentes nocivos ou  multiplos vinculos.
        /* Verificar tabela de ocorrencias
         * @todo Fazer validacao das Ocorrencias;
         *       Passado BRANCO pois a maioria dos trabalhadores sao da categoria '01';
         */
        $ocorrencia = $this->_vinculo->ocorrencia; //parent::CARACTER_BRANCO;
        $this->_atribuiOcorrencia($ocorrencia);

        // Bloco 20 | 202-216 | Contribuicao do segurado.
        // @todo Fazer validacao das contribuicoes.
        $contribuicao = parent::CARACTER_ZERO;
        if ($remuneracao[RegistroSEFIP::SEFIP_DEDUCAO_INSS_PATRONAL] > 0) {
            $this->_atribuiContribuicaoINSSInformada($remuneracao[RegistroSEFIP::SEFIP_VALOR_DESC_SEGURADO]);
        } else {
            $this->_atribuiContribuicaoINSSInformada($remuneracao[RegistroSEFIP::SEFIP_CONTRIB_INSS_DESC]);
        }

        // Bloco 21 | 217-231 | Base de calculo da contribuicao previdenciaria.
        // @todo Verificar validacao.
        if ($remuneracao[RegistroSEFIP::SEFIP_DEDUCAO_INSS_PATRONAL] > 0) {
            $this->_atribuiBaseGPS($remuneracao[RegistroSEFIP::SEFIP_CONTRIB_PREVIDENCIA], true);
        } else {
            $this->_atribuiBaseGPS($remuneracao[RegistroSEFIP::SEFIP_REM_BASE_GPS]);
        }

        // Bloco 22 | 232-246 | Base de Calculo 13.
        // @todo Verificar validacao.
        $this->_atribuiBase13($remuneracao[RegistroSEFIP::SEFIP_REM_13SAL_MOV] + $remuneracao[RegistroSEFIP::SEFIP_REM_13SAL_MOV_13] + $remuneracao[RegistroSEFIP::SEFIP_COMPLEM_INSS_13SAL]);

        // Bloco 23 | 247-261 | Base de Calculo 13 GPS.
        // @todo Verificar validacao.
        $this->_atribuiBase13GPS($remuneracao[RegistroSEFIP::SEFIP_REM_13SAL_GPS], $remuneracao[RegistroSEFIP::SEFIP_COMPLEM_INSS_13SAL]);

        // Bloco 31 | 262-359 | Brancos. Preencher com ' '.
        $this->atribuiDado(parent::CARACTER_VAZIO, 98, parent::CARACTER_BRANCO);

        // Bloco 32 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);

    }


    public function setCodMovimentacao($codMovimentacao)
    {
        $this->_codMovimentacao = $codMovimentacao;
    }


    public function setNrCategoria($nrCategoria)
    {
        $this->_nrCategoria = $nrCategoria;
    }


    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }


    public function setNrInscricaoTomador($nrInscricaoTomador)
    {
        $this->_nrInscricaoTomador = $nrInscricaoTomador;
    }


    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }


    public function setTpInscricaoTomador($tpInscricaoTomador)
    {
        $this->_tpInscricaoTomador = $tpInscricaoTomador;
    }


    /**
     * Atribui Data de Admissao.
     *
     * @param string $dtAdmissao Data de Admissao.
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     */
    private function _atribuiDataAdmissao($dtAdmissao)
    {

        // Obrigatorio para as categorias de trabalhadores 01, 03, 04, 05, 06, 07, 11, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '03', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
            
            $dataAdmissao = DateUtil::getDataDeString($dtAdmissao);
            $dataAdmissao = $dataAdmissao->format('d/m/Y');
            
        } else {
            $dataAdmissao = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataAdmissao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataAdmissao);
    }


    /**
     * Atribui Matricula.
     *
     * @param string $matricula Matricula.
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     */
    private function _atribuiMatricula($cdmatricula)
    {

        $matricula = preg_replace("/[^0-9\s]/", "", $cdmatricula);

        // Nao pode ser informado para as categorias 06, 13, 14, 15, 16 , 17, 18, 22, 23, 24 ou 25.
        $codCategorias = array(
            '06', '13', '14', '15', '16', '17', '18', '22', '23', '24', '25',
        );

        if (array_search($this->_nrCategoria, $codCategorias) !== false) {
            $matricula = parent::CARACTER_BRANCO;
        }

        if ((empty($matricula) === true) || ($matricula === parent::CARACTER_BRANCO)) {
            $this->atribuiDado($matricula, 11, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        } else {
            $this->atribuiDado($matricula, 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        }
    }


    /**
     * Atribui CTPS.
     *
     * @param string $nrCTPS Numero da Carteira Profissional.
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     *         leonardocarvalho - 28/05/2014 - Condição para ignorar tipo de vinculo SOCIO
     */
    private function _atribuiCTPS($nrCTPS)
    {
        $nrtipovinculo = $this->_vinculo->getNrtipovinculom();

        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $nrCTPS = parent::CARACTER_BRANCO;
        }

        if (((empty($nrCTPS) === true) || ($nrCTPS === parent::CARACTER_BRANCO )) && ($nrtipovinculo != self::SOCIO) && ($nrtipovinculo != self::AUTONOMO)) {
            //@todo MsgCalculo_SEFIP
            MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $this->_pessoa->historico->getNmpessoa() . ' não possui numero CTPS.', MsgCalculo::ERRO);
            //throw new Exception('Vínculo ' . $this->_pessoa->historico->getNmpessoa() . ' não possui numero CTPS.');
        } else {
            if ($nrtipovinculo != self::SOCIO && $nrtipovinculo != self::AUTONOMO)  {
                $this->atribuiDado($nrCTPS, 7, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
            } else {
                $this->atribuiDado($nrCTPS, 7, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
            }
        }
    }

    private function _atribuiSerieCTPS($serieCTPS)
    {
        if (!ctype_digit($serieCTPS)) {
            $serieCTPS = '00001';
        }

        $nrtipovinculo = $this->_vinculo->getNrtipovinculom();


        if (((empty($serieCTPS) === true) || ($serieCTPS === parent::CARACTER_BRANCO)) && ($nrtipovinculo != self::SOCIO) && ($nrtipovinculo != self::AUTONOMO)) {
            MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $this->_pessoa->historico->getNmpessoa() . ' não possui numero de serie CTPS.', MsgCalculo::ERRO);
            $this->atribuiDado($serieCTPS, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        } else {
            $codCategorias = array(
                '01', '02', '03', '04', '06', '07', '26',
            );

            if ((array_search($this->_nrCategoria, $codCategorias) === false)) {
                $serieCTPS = parent::CARACTER_BRANCO;
            }

            if (($nrtipovinculo != self::SOCIO) && ($nrtipovinculo != self::AUTONOMO)) {
                $this->atribuiDado($serieCTPS, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
            } else {
                $this->atribuiDado($serieCTPS, 5, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
            }
        }
    }

    private function _atribuiDataOpcaoFGTS($dtOpcao)
    {
        $codCategorias = array(
            '01', '03', '04', '05', '06', '07',
        );

        if ((array_search($this->_nrCategoria, $codCategorias) === false)
            || ($this->_nrRecolhimento === '640')
        ) {
            $dataOpcao = parent::CARACTER_BRANCO;
        } else {
            $dataOpcao = DateUtil::getDataDeString($dtOpcao);
            $dataOpcao = $dataOpcao->format('d/m/Y');
        }

        $this->atribuiDado($dataOpcao, 8, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
        unset($dataOpcao);
    }

    private function _atribuiDataNascimento($dtNascimento)
    {

        // Campo obrigatorio para as categorias de trabalhadores 01, 02, 03, 04, 05, 06, 07, 12, 19, 20, 21 e 26.
        $codCategorias = array(
            '01', '02', '03', '04', '05', '06', '07', '12', '19', '20', '21', '26',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $dataNascimento = parent::CARACTER_BRANCO;
        } elseif(is_null($dtNascimento)) {
            $dataNascimento = parent::CARACTER_BRANCO;
            //@todo MsgCalculo_SEFIP
            MsgCalculo::getInstance()->adicionaMensagem('Vínculo ' . $this->_pessoa->historico->getNmpessoa() . ' não possui data de nascimento.');
        } else {
            $dataNascimento = DateUtil::getDataDeString($dtNascimento);
            $dataNascimento = $dataNascimento->format('d/m/Y');
        }

        $this->atribuiDado($dataNascimento, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        unset($dataNascimento);
    }


    /**
     * Atribui codigo CBO.
     *
     * @param string $cbo Codigo CBO.
     *
     * @return void
     *
     * @author alessandrobartels - 15/05/2012
     */
    private function _atribuiCbo($cbo)
    {
        /* Utilizar os quatro primeiros dígitos do grupo “Família” do novo CBO, acrescentando zero a
         * esquerda.(0 + XXXX onde XXXX é o código da família do novo CBO a qual pertence o
         * trabalhador).
         */
        if (strlen($cbo) > 4) {
            $cbo = substr($cbo, 0, 4);
        }

        $this->atribuiDado($cbo, '5', parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }

    /**
     * Atribui Remuneracao sem 13.
     *
     * @param string $remuneracao Valor da remuneracao sem 13 salario.
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     */
    private function _atribuiRemuneracao($remuneracao)
    {
        /* Campo obrigatorio para as categorias 05, 11, 13, 14, 15, 16, 17, 18, 22, 23, 24 e 25.
         * Opcional para as categorias 01, 02, 03, 04, 06, 07, 12, 19, 20, 21 e 26.
         * Nao pode ser informado para a competencia 13.
         * Obs.: Abrange todas as categorias portanto nao precisa de validacao.
         */
        /*$codRecolhimentos = array(
            '650'
        );
        if (array_search($this->_nrRecolhimento, $codRecolhimentos) !== false) {
            $remuneracao = parent::CARACTER_ZERO;
        }*/

        if ($this->_competencia13 === true) {
            $remuneracao = parent::CARACTER_ZERO;
        } /*else {
            $remuneracao = number_format($remuneracao, 2, '.', '');
//            $remuneracao = str_replace(self::CARACTER_PONTO, self::CARACTER_VAZIO, $remuneracao);
        }*/

        $this->atribuiDado($remuneracao, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui Remuneracao 13 - Parcela 13 pago no mes.
     *
     * @param string $remuneracao13 Valor da remuneracao 13 pago no mes.
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     */
    private function _atribuiRemuneracao13($remuneracao13)
    {
        /* Nao pode ser informado para a competencia 13.
         * Campo obrigatorio para categoria 02.
         * Campo opcional para as categorias de trabalhadores 01, 03, 04, 06, 07, 12, 19, 20, 21 e 26.
         */
        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '12', '19', '20', '21', '26',
        );

        if ((array_search($this->_nrCategoria, $codCategorias) === false)
            || ($this->_competencia13 === true)
        ) {
            $remuneracao13 = parent::CARACTER_ZERO;
        }
        /* else {
                    $remuneracao = number_format($remuneracao, 2, '.', '');
        //            $remuneracao13 = str_replace(self::CARACTER_PONTO, self::CARACTER_VAZIO, $remuneracao13);
                }*/

        $this->atribuiDado($remuneracao13, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui Ocorrencias a agentes nocivos ou multiplos vinculos.
     *
     * @param string $ocorrencia Ocorrencia a agente nocivos ou multiplos vinculos.
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     */
    private function _atribuiOcorrencia($ocorrencia)
    {
        /* @todo - Falta implementacao do campo para a descricao do agente nocivo.
         * Campo opcional para as categorias 01, 02, 03, 04, 05, 06, 07, 11, 12, 13, 15, 17, 18, 19, 20, 21, 22, 23, 24 e 25.
         * Deve ficar em branco se trabalhador não esteve exposto a agente nocivo e não possui mais de um vínculo empregatício.
         * Para empregado doméstico (Cat 06) e diretor não empregado (Cat 05) permitido apenas branco ou 05.
         * Para as categorias 02, 22 e 23 permitido apenas branco, 01, 02, 03 e 04.
         * Obrigatório para categoria 26, devendo ser informado 05, 06, 07 ou 08.
         * Não pode ser informado para as demais categorias.
         * Deve ser uma ocorrência válida (ver tabela).
         * Sempre que não informado o campo deve ficar em branco.
         */

        $codCategorias = array(
            '01', '02', '03', '04', '06', '07', '11', '12', '13', '15', '17', '18', '19', '20', '21', '22', '23', '24', '25',
        );

        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $ocorrencia = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($ocorrencia, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

    }


    /**
     * Atribui Contribuicao descontada do segurado para o INSS.
     *
     * @param string $contribuicao Valor da Contribuicao do segurado.
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     */
    private function _atribuiContribuicaoINSSInformada($contribuicao)
    {
        /*
         * @todo Fazer validacao da contribuicao.
         *  Campo opcional para as ocorrências 05, 06, 07 e 08.
         *  Campo opcional para as categorias de trabalhadores igual a 01, 02, 04, 06, 07, 12, 19, 20, 21 e 26.
         *  Campo opcional para as categorias de trabalhadores igual a 05, 11, 13, 15, 17, 18, 24 e 25 a partir da competência 04/2003
         *  Campo opcional para os codigos de recolhimento 130, 135 e 650.
         *  Campo opcional para competencia maior ou igual a 12/1999 para afastamentos por motivo de licenca-maternidade
         *  Iniciada a partir de 01/12/1999 e com beneficios requeridos ate 31/08/2003.
         */

        $codOcorrencias = array(
            '05', '06', '07', '08'
        );

        $codRecolhimentos = array(
            '130', '135', '650'
        );

        if ((array_search($this->_vinculo->ocorrencia, $codOcorrencias) !== false) ||
            (array_search($this->_nrRecolhimento, $codRecolhimentos) !== false)) {
            $contribuicao = $contribuicao;
        } else {
            $contribuicao = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($contribuicao, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui Base GPS.
     *
     * @param string $baseGPS Valor da Base GPS.
     * @param boolean $forcaValor Parâmetro passo para os casos em que existe afastamento 15 dias empregador e a SEFIP encontrou para passar o valor da Contribuição Previdenciária [28/01/2021]
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     */
    private function _atribuiBaseGPS($baseGPS, $forcaValor=false)
    {
        /* @todo Campo obrigatorio para as movimentacoes (registro 32) por O1, O2, R, Z2, Z3 e Z4.
         * @todo Campo obrigatorio quando o Indicativo 'C' de Recolhimento Complementar de FGTS for informado (registro
         *       Tipo 32 - campo 12).
         *
         * - Campo opcional para as categorias 01, 02, 04, 05, 06, 07,11, 12, 19, 20, 21 e 26.
         * - Nao pode ser informado na competencia 13.
         * - Nao pode ser informado para os codigos de recolhimento 145, 307, 317, 327, 337, 345, 640 e 660.
         * - Sempre que nao informado preencher com zeros.
         */

        $codRecol = array(
            '145', '307', '317', '327', '337', '345', '640', '660',
        );

        $codCategorias = array(
            '01', '02', '04', '05', '06', '07', '11', '12', '19', '20', '21', '26',
        );

        $codMovimentacoes = array(
            'O1', 'O2', 'R', 'Z2', 'Z3', 'Z4',
        );

        if (!$forcaValor) {
            if ((array_search($this->_nrRecolhimento, $codRecol) !== false)
                || (array_search($this->_nrCategoria, $codCategorias) === false)
                || (array_search($this->_codMovimentacao, $codMovimentacoes) === false)
                || ($this->_competencia13 === true)
            ) {
                $baseGPS = parent::CARACTER_ZERO;
            }
        }

        $this->atribuiDado($baseGPS, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

    }


    /**
     * Atribui Base de 13 salario.
     *
     * @param string $base13 Valor da Base para calculo de 13 salario.
     *
     * @return void
     *
     * @author alessandrobartels - 01/02/2012
     */
    private function _atribuiBase13($base13)
    {

        //@todo - Rever as validacoes.
        //a - Obrigatorio para a competencia 13.
        //b - Obrigatorio no mes de rescisao para quem trabalhou mais de 15 dias no ano e possui c�digo de movimenta��o por
        //    motivo rescis�o (exceto rescis�o com justa causa), aposentadoria com quebra de v�nculo ou falecimento.
        //c - Obrigat�rio para os c�digos de recolhimento 130 e 135
        //d - Obrigat�rio para os c�digos de recolhimento 608 quando houver trabalhador da categoria 02 no arquivo.
        //e - Opcional para os c�digos de recolhimento 650.
        //f - So pode ser informado para as categorias 01, 02, 04, 06, 07, 12, 19, 20, 21 e 26.
        //g - Campo opcional para as categorias 01, 04, 06, 07, 12, 19, 20, 21 e 26 na compet�ncia 12.
        //h - Sempre que n�o informado preencher com zeros.
        //PEGAR BASE DO ITEMCALCULO.

        $codCategorias = array('01', '02', '04', '06', '07', '12', '19', '20', '21', '26',);
        $codRecol = array('130', '135', '650');
        $codMovimentacoes = array('I1', 'I2', 'I3', 'I4', 'I5', 'J', 'K', 'L');

        //ver letra 'f'
        if (array_search($this->_nrCategoria, $codCategorias) === false) {
            $base13 = parent::CARACTER_ZERO;
        } else {

            //ver letra 'a','c','e'
            if (($this->_competencia13 === false)
                || (array_search($this->_nrRecolhimento, $codRecol) === false)
            ) {

                //ver letra 'b'
                if ($this->_dtrescisao) {

                    if ($this->_dtrescisao < $this->_dtadmissao) {
                        $base13 = parent::CARACTER_ZERO;
                    }

                }

                /* verificar rescisao  - Movimentacoes*/
                if ($this->_competencia === self::CARACTER_DOZE) {
                    true;
                    // Verifica competencia 12. Segundo ARTE deve passar o complemento do 13 se houver.
                    /*  $base13 = //complemento de 13 salario se houver;*/
                }
            }
        }

        $this->_base13 = $base13;
        $this->atribuiDado($base13, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui Base do 13 salario para GPS.
     *
     * @param string $base13GPS Valor da Base do 13 salario para GPS.
     *
     * @return void
     *
     * @author alessandrobartels - 02/02/2012
     */
    private function _atribuiBase13GPS($base13GPS, $complInss)
    {
        /* @todo Verificar validacao.
         * . Campo opcional para a compet�ncia 12.
         * � N�o pode ser informado nas demais compet�ncias.
         * � Campo opcional para as categorias 01, 04 , 06, 07, 12, 19, 20, 21 e 26.
         * � Se informado o campo 22 (registro 30) deve ser diferente de zeros.
         * � Sempre que n�o informado preencher com zeros.
         *
         *
         * PEGAR BASE DO ITEMCALCULO.
         * */
        //Artur 09/01/2018 Para competencia 13 gerar a base 13gps
        if($this->_competencia == self::CARACTER_DOZE){
            $base13GPS += $complInss;
        }

        if ($this->_base13 > 0) {
            $codCategorias = array(
                '01', '04', '06', '07', '12', '19', '20', '21', '26',
            );

            if ((array_search($this->_nrCategoria, $codCategorias) === false)
                && (($this->_competencia !== self::CARACTER_DOZE)
                    || ($this->_competencia13 === true))
            ) {
                $base13GPS = parent::CARACTER_ZERO;
            }
        } else {
            $base13GPS = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($base13GPS, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }
}