<?php
namespace HCM\Geral\Controllers;

use Teknisa\Geral\Util\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;

/**
 * @TODO: The php does not overload methods,
 * @TODO: So I had to rewrite an specific crud class for the TecFood.
 * @TODO: Later, we will create a more beautiful solution ..
 */
abstract class TecfoodCrud /* extends Crud  */
{

	/** @var Environment */
	protected $environment;
	/** @var Manager */
	protected $dataSourceManager;
	protected $dataSourceName;
	protected $dataSourceNameToSave  = false;
    /** @var ParameterBag */
    protected $parameterBag;
    /** @var string[] */
    protected $parameterBagColumns = array();

    public function __construct(Manager $dataSourceManager, Environment $environment, ParameterBag $parameterBag) {
		$this->dataSourceManager = $dataSourceManager;
		$this->environment = $environment;
        $this->parameterBag = $parameterBag;
	}

	public function save(DTO\Request $request, DTO\Response $response) {
		$rows = array();
		$request = $this->buildDataSetIfRow($request);
		foreach ($request->getDataSet()->getRows() as $row) {

			$currentTime = $this->environment->getCurrentTime()->format("d/m/Y");

			if ($row['__is_new']) {
				if (isset($row['NRORGINCLUSAO'])) $row['NRORGINCLUSAO'] = $this->environment->getNrOrgTrab();
				if (isset($row['DTINCLUSAO'])) $row['DTINCLUSAO'] = $currentTime;
                if (isset($row['CDOPERINCLUSAO'])) $row['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
				if (isset($row['NRORG'])) $row['NRORG'] = $this->environment->getNrOrgTrab();
			}

			if (isset($row['IDATIVO'])) $row['IDATIVO'] = $row['IDATIVO'] ?: 'S';
			if (isset($row['CDOPERULTATU'])) $row['CDOPERULTATU'] = $this->environment->getCdOperador();
			if (isset($row['DTULTATU'])) $row['DTULTATU'] = $currentTime;
			if (isset($row['NRORGULTATU'])) $row['NRORGULTATU'] = $this->environment->getNrOrgTrab();
			if (isset($row['DTULTATUPRFIL'])) $row['DTULTATUPRFIL'] = $currentTime;

			$rows[] = $row;
		}
		$dataSourceName =  $this->dataSourceNameToSave ?: $this->dataSourceName;
		$dataSet = new DataSet($dataSourceName, $rows);
		$persistedRows = $this->dataSourceManager->persist($dataSet);
        $response->addDataSet(new DataSet($dataSourceName, $persistedRows));
		$nrPersistedRows = count($persistedRows);
		$response->addMessage(new Message("{$nrPersistedRows} rows persisted with success"));
	}

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
        //adiciona condição que filtra automaticamente pela NRORG logada
        // $filterCriteria->addCondition("NRORG", "=", $this->environment->getNrOrgTrab());
		$filterCriteria->setDataSourceName($this->dataSourceName);
        $newFilterCriteria = $this->populateParameterBag($filterCriteria);
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		$response->addDataSet($dataSet);
	}

	public function delete(DTO\Request $request, DTO\Response $response) {
		$request = $this->buildDataSetIfRow($request);
		$dataSet = $request->getDataSet();
		$dataSet->setDataSourceName($this->dataSourceName);
		$deletedRows = $this->dataSourceManager->delete($dataSet);
		$response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
		$nrDeletedRows = count($deletedRows);
		$response->addMessage(new Message("{$nrDeletedRows} row(s) deleted."));
	}

	protected function buildDataSetIfRow(DTO\Request $request) {
		if ($request instanceof DTO\Request\Row) {
			$dataSet = new DataSet($this->dataSourceName, array((array)$request->getRow()));
			$request = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
		}
		return $request;
	}

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        return $newFilterCriteria;
    }
}