<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\ESocialRepository;
use HCM\Geral\Services\UploadFileService;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;

class EsoParametroService {

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ESocialRepository $esocialRepository, UploadFileService $uploadsFileService) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->eSocialRepository = $esocialRepository;
        $this->uploadsFileService = $uploadsFileService;
    }

    public function setDadosCertificado($row) {
        try {
            $esoParametro = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->findOneBy(
                array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nrparametro' => $row['NRPARAMETRO'])
            );
            
            // Apaga o arquivo antigo do FileServer caso exista
            if ($esoParametro->getDsarqucert()) {
                $this->uploadsFileService->deleteUploadFile($esoParametro->getDsarqucert());
            }
            
            // Insere o novo arquivo
            $upload = $this->uploadsFileService->setGpeAnexos($row);
            
            // Atualiza o caminho do arquivo e a senha no parametro
            $esoParametro->setDsarqucert($row['DSPATH'][0]["path"]);
            $esoParametro->setDssenhacert($row['DSSENHACERT']);
            $esoParametro->setDtultatu(DateUtil::getDataAtual());
            $esoParametro->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $esoParametro->setCdoperultatu($this->hcmParameters->getCdoperador());
            $this->entityManager->persist($esoParametro);
            $this->entityManager->flush();
            
            return $upload;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaVersaoNova($row) {
        try {
            $esoParametro = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->findOneBy(array('nrorg' => $row['NRORG'], 'nrparametro' => $row['NRPARAMETRO']));
            
            if (is_object($esoParametro)) {
                if ($esoParametro->getDsversao() != $row['DSVERSAO']) {
                    $this->entityManager->beginTransaction();
                    $update = $this->eSocialRepository->updateEnvioVersao($row['NRORG'], $row['NRESTRUTURAM'], $row['DSVERSAO']);
                    if ($update) {
                        $this->entityManager->commit();
                    } else {
                        throw new \Exception(Messages::SAVE_ERROR_MESSAGE);
                    }
                }
            }
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

}